@@ -94,7 +94,7 @@ BASE_VERSION = 1
 # v1.54: Add events support.
 # v1.55: Add deploy templates API.
 # v1.56: Add support for building configdrives.
-# v1.57: Add support for updating an exisiting allocation.
+# v1.57: Add support for updating an existing allocation.
 # v1.58: Add support for backfilling allocations.
 # v1.59: Add support vendor data in configdrives.
 # v1.60: Add owner to the allocation object.
@@ -126,6 +126,8 @@ BASE_VERSION = 1
 # v1.86: Add firmware interface
 # v1.87: Add service verb
 # v1.88: Add name field to port.
+# v1.89: Add API for attaching/detaching virtual media
+# v1.90: Accept ovn vtep switch metadata schema to port.local_link_connection
 
 MINOR_0_JUNO = 0
 MINOR_1_INITIAL_VERSION = 1
@@ -216,6 +218,9 @@ MINOR_85_UNHOLD_VERB = 85
 MINOR_86_FIRMWARE_INTERFACE = 86
 MINOR_87_SERVICE = 87
 MINOR_88_PORT_NAME = 88
+MINOR_89_ATTACH_DETACH_VMEDIA = 89
+MINOR_90_OVN_VTEP = 90
+MINOR_91_DOT_JSON = 91
 
 # When adding another version, update:
 # - MINOR_MAX_VERSION
@@ -223,7 +228,7 @@ MINOR_88_PORT_NAME = 88
 #   explanation of what changed in the new version
 # - common/release_mappings.py, RELEASE_MAPPING['master']['api']
 
-MINOR_MAX_VERSION = MINOR_88_PORT_NAME
+MINOR_MAX_VERSION = MINOR_91_DOT_JSON
 
 # String representations of the minor and maximum versions
 _MIN_VERSION_STRING = '{}.{}'.format(BASE_VERSION, MINOR_1_INITIAL_VERSION)
