@@ -26,6 +26,7 @@ from oslo_utils import uuidutils
 from ironic.api.controllers import base as api_base
 from ironic.api.controllers import v1 as api_v1
 from ironic.api.controllers.v1 import ramdisk
+from ironic.common import exception
 from ironic.common import states
 from ironic.conductor import rpcapi
 from ironic.drivers.modules import inspect_utils
@@ -80,6 +81,9 @@ class TestLookup(test_api_base.BaseApiTest):
             'heartbeat_timeout': CONF.api.ramdisk_heartbeat_timeout,
             'agent_token': mock.ANY,
             'agent_token_required': True,
+            'agent_md5_checksum_enable': CONF.agent.allow_md5_checksum,
+            'disable_deep_image_inspection': CONF.conductor.disable_deep_image_inspection,  # noqa
+            'permitted_image_formats': CONF.conductor.permitted_image_formats,
         }
         self.assertEqual(expected_config, data['config'])
         self.assertIsNotNone(data['config']['agent_token'])
@@ -202,7 +206,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
     def test_old_api_version(self):
         response = self.post_json(
             '/heartbeat/%s' % uuidutils.generate_uuid(),
-            {'callback_url': 'url'},
+            {'callback_url': 'https://url'},
             headers={api_base.Version.string: str(api_v1.min_version())},
             expect_errors=True)
         self.assertEqual(http_client.NOT_FOUND, response.status_int)
@@ -210,7 +214,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
     def test_node_not_found(self):
         response = self.post_json(
             '/heartbeat/%s' % uuidutils.generate_uuid(),
-            {'callback_url': 'url'},
+            {'callback_url': 'https://url'},
             headers={api_base.Version.string: str(api_v1.max_version())},
             expect_errors=True)
         self.assertEqual(http_client.NOT_FOUND, response.status_int)
@@ -220,29 +224,30 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'x'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None, 'x',
-                                               None, None, None,
+                                               node.uuid, 'https://url', None,
+                                               'x', None, None, None,
                                                topic='test-topic')
 
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_ok_with_json(self, mock_heartbeat):
+        headers = {api_base.Version.string: '1.90'}
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s.json' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'maybe some magic'},
-            headers={api_base.Version.string: str(api_v1.max_version())})
+            headers=headers)
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
-                                               'maybe some magic',
+                                               node.uuid, 'https://url',
+                                               None, 'maybe some magic',
                                                None, None, None,
                                                topic='test-topic')
 
@@ -251,13 +256,13 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context, name='test.1')
         response = self.post_json(
             '/heartbeat/%s' % node.name,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'token'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
+                                               node.uuid, 'https://url', None,
                                                'token', None, None, None,
                                                topic='test-topic')
 
@@ -266,14 +271,15 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_version': '1.4.1',
              'agent_token': 'meow'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', '1.4.1',
+                                               node.uuid, 'https://url',
+                                               '1.4.1',
                                                'meow',
                                                None, None, None,
                                                topic='test-topic')
@@ -283,20 +289,31 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_version': '1.4.1'},
             headers={api_base.Version.string: '1.35'},
             expect_errors=True)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
 
+    @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
+    def test_heartbeat_rejects_file_url(self, mock_heartbeat):
+        node = obj_utils.create_test_node(
+            self.context)
+        response = self.post_json(
+            '/heartbeat/%s' % node.uuid,
+            {'callback_url': 'file:///path/to/the/wizzard'},
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_heartbeat_rejects_different_callback_url(self, mock_heartbeat):
         node = obj_utils.create_test_node(
             self.context,
-            driver_internal_info={'agent_url': 'url'})
+            driver_internal_info={'agent_url': 'https://url'})
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url2'},
+            {'callback_url': 'https://url2'},
             headers={api_base.Version.string: str(api_v1.max_version())},
             expect_errors=True)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
@@ -306,13 +323,13 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'http://url',
              'agent_token': 'abcdef1'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
+                                               node.uuid, 'http://url', None,
                                                'abcdef1', None, None, None,
                                                topic='test-topic')
 
@@ -321,14 +338,14 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_verify_ca': 'abcdef1'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
+                                               node.uuid, 'https://url', None,
                                                'meow', 'abcdef1', None, None,
                                                topic='test-topic')
 
@@ -337,7 +354,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_status': 'start',
              'agent_status_message': 'woof',
@@ -346,7 +363,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
+                                               node.uuid, 'https://url', None,
                                                'meow', 'abcdef1', 'start',
                                                'woof', topic='test-topic')
 
@@ -355,7 +372,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_status': 'invalid_state',
              'agent_status_message': 'woof',
@@ -369,7 +386,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_verify_ca': 'abcd'},
             headers={api_base.Version.string: '1.67'},
@@ -381,7 +398,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_verify_ca': 'abcd',
              'agent_status': 'wow',
@@ -456,6 +473,19 @@ class TestContinueInspection(test_api_base.BaseApiTest):
             mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
             plugin_data={'test': 42}, topic='test-topic')
 
+    def test_bmc_address_as_none(self, mock_lookup, mock_continue):
+        mock_lookup.return_value = self.node
+        self.inventory['bmc_address'] = None
+        self.inventory['bmc_v6address'] = None
+        response = self.post_json('/continue_inspection', self.data)
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual({'uuid': self.node.uuid}, response.json)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, [], node_uuid=None)
+        mock_continue.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
+            plugin_data={'test': 42}, topic='test-topic')
+
     @mock.patch.object(rpcapi.ConductorAPI, 'get_node_with_token',
                        autospec=True)
     def test_new_api(self, mock_get_node, mock_lookup, mock_continue):
@@ -524,3 +554,82 @@ class TestContinueInspectionScopedRBAC(TestContinueInspection):
         cfg.CONF.set_override('enforce_new_defaults', True,
                               group='oslo_policy')
         cfg.CONF.set_override('auth_strategy', 'keystone')
+
+
+@mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for', autospec=True,
+                   return_value='test-topic')
+@mock.patch.object(rpcapi.ConductorAPI, 'create_node', autospec=True)
+@mock.patch.object(rpcapi.ConductorAPI, 'continue_inspection', autospec=True)
+@mock.patch.object(inspect_utils, 'lookup_node', autospec=True,
+                   side_effect=inspect_utils.AutoEnrollPossible)
+class TestContinueInspectionAutoDiscovery(test_api_base.BaseApiTest):
+
+    def setUp(self):
+        super().setUp()
+        CONF.set_override('enabled', True, group='auto_discovery')
+        CONF.set_override('driver', 'fake-hardware', group='auto_discovery')
+        self.addresses = ['11:22:33:44:55:66', '66:55:44:33:22:11']
+        self.bmcs = ['192.0.2.42']
+        self.inventory = {
+            'bmc_address': self.bmcs[0],
+            'interfaces': [
+                {'mac_address': mac, 'name': f'em{i}'}
+                for i, mac in enumerate(self.addresses)
+            ],
+        }
+        self.data = {
+            'inventory': self.inventory,
+            'test': 42,
+        }
+        self.node = obj_utils.get_test_node(self.context,
+                                            uuid=uuidutils.generate_uuid(),
+                                            provision_state='enroll')
+
+    def test_enroll(self, mock_lookup, mock_continue, mock_create,
+                    mock_get_topic):
+        mock_create.return_value = self.node
+        response = self.post_json('/continue_inspection', self.data)
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual({'uuid': self.node.uuid}, response.json)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, self.bmcs, node_uuid=None)
+        mock_continue.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
+            plugin_data={'test': 42, 'auto_discovered': True},
+            topic='test-topic')
+        new_node = mock_create.call_args.args[2]  # create(self, context, node)
+        self.assertEqual('fake-hardware', new_node.driver)
+        self.assertIsNone(new_node.resource_class)
+        self.assertEqual('', new_node.conductor_group)
+        self.assertEqual('enroll', new_node.provision_state)
+
+    def test_wrong_driver(self, mock_lookup, mock_continue, mock_create,
+                          mock_get_topic):
+        mock_get_topic.side_effect = exception.NoValidHost()
+        response = self.post_json(
+            '/continue_inspection', self.data,
+            expect_errors=True)
+        self.assertEqual(http_client.INTERNAL_SERVER_ERROR,
+                         response.status_int)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, self.bmcs, node_uuid=None)
+        mock_create.assert_not_called()
+        mock_continue.assert_not_called()
+
+    def test_override_defaults(self, mock_lookup, mock_continue, mock_create,
+                               mock_get_topic):
+        CONF.set_override('default_resource_class', 'xlarge-1')
+        # TODO(dtantsur): default_conductor_group
+        mock_create.return_value = self.node
+        response = self.post_json('/continue_inspection', self.data)
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, self.bmcs, node_uuid=None)
+        mock_continue.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
+            plugin_data={'test': 42, 'auto_discovered': True},
+            topic='test-topic')
+        new_node = mock_create.call_args.args[2]  # create(self, context, node)
+        self.assertEqual('fake-hardware', new_node.driver)
+        self.assertEqual('xlarge-1', new_node.resource_class)
+        self.assertEqual('', new_node.conductor_group)
